/*
 * This file is part of source for dviviewer, 
 * a DVI file viewer for Qtopia environment. 
 * 
 * Copyright (C) 2003  hn43o.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "binaryfile.h"

BinaryFile::BinaryFile() {
  isValid = FALSE;
}
BinaryFile::~BinaryFile() {
  if ( file.isOpen() )
    file.close();
}
bool BinaryFile::open( QString fileName ) {
  if ( isValid )
    return FALSE;
  file.setName( fileName );
  if ( !file.open( IO_ReadOnly ) )
    return FALSE;
  return TRUE;
}

void BinaryFile::close() {
  if ( file.isOpen() )
    file.close();
  isValid = FALSE;
}

uint BinaryFile::getUInt1() {
  return file.getch();
}
int BinaryFile::getInt1() {
  return ( signed char ) file.getch();
}
unsigned int BinaryFile::getUInt2() {
  unsigned char c0, c1;
  c0 = file.getch();
  c1 = file.getch();
  return ( c0 << 8 ) + c1;
}
int BinaryFile::getInt2() {
  unsigned char c0, c1;
  c0 = file.getch();
  c1 = file.getch();
  uint tmp = ( c0 << 8 ) + c1;
  return ( ( tmp & ( 1 << 15 ) ) ? ( int ) ( tmp | ( 0xffff << 16 ) ) : tmp );
}
unsigned int BinaryFile::getUInt3() {
  unsigned char c0, c1, c2;
  c0 = file.getch();
  c1 = file.getch();
  c2 = file.getch();
  return ( c0 << 16 ) + ( c1 << 8 ) + c2;
}
int BinaryFile::getInt3() {
  unsigned char c0, c1, c2;
  c0 = file.getch();
  c1 = file.getch();
  c2 = file.getch();
  uint tmp = ( c0 << 16 ) + ( c1 << 8 ) + c2;
  return ( ( tmp & ( 1 << 23 ) ) ? ( int ) ( tmp | ( 0xff << 24 ) ) : tmp );
}
unsigned int BinaryFile::getUInt() {
  unsigned char c0, c1, c2, c3;
  c0 = file.getch();
  c1 = file.getch();
  c2 = file.getch();
  c3 = file.getch();
  return ( c0 << 24 ) + ( c1 << 16 ) + ( c2 << 8 ) + c3;
}
int BinaryFile::getInt() {
  unsigned char c0, c1, c2, c3;
  c0 = file.getch();
  c1 = file.getch();
  c2 = file.getch();
  c3 = file.getch();
  return ( int ) ( ( c0 << 24 ) + ( c1 << 16 ) + ( c2 << 8 ) + c3 );
}
