/*
 * This file is part of source for dviviewer, 
 * a DVI file viewer for Qtopia environment. 
 * 
 * Copyright (C) 2003  hn43o.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef DVIPAINTER_H
#define DVIPAINTER_H

#include <qdict.h>
#include <qmap.h>
#include <qpainter.h>
#include <qrect.h>
#include <qstring.h>
#include "texfont.h"

class DviPainter {
 public:
  DviPainter();
  virtual ~DviPainter();

  // draw everything.
  virtual void draw( QRect ) = 0;

  // set logical geometry.
  virtual void setGeometry( uint scaleFactor, uint designSize, 
						   uint magnification );
  virtual void clear() = 0;
  virtual void setFont( QString, int, int ) = 0; 
  virtual void drawBox( int x, int y, int w, int h ) = 0;
  virtual uint drawChar( int x, int y, uint code ) = 0; // returns width.

 protected:
  uint _scaleFactor, _designSize, _magnification;
};

class DviViewPainter : public DviPainter {
 public:
  DviViewPainter();
  virtual void draw( QRect );
  virtual void setGeometry( uint scaleFactor, uint designSize, 
						   uint magnification );
  virtual void clear();
  virtual void setFont( QString, int, int );
  virtual void drawBox( int x, int y, int w, int h );
  virtual uint drawChar( int x, int y, uint code );

#if 0
  QSize size();
#endif

  // set 'physical' geometry. w, h are in pixels, paperW is in inches.
  void setDpi( uint );
  void setPainter( QPainter * );
  void clearFonts();

 private:
  typedef struct {
	int x, y;
	QRect rect;
	uint code;
	TexFont *font;
  } Char;
  typedef struct {
	int x, y, w, h;
  } Box;
  QArray<Char> chars;
  QArray<Box> boxen;

  QPainter *p;
  uint dpi;
  uint curX, curY;
  QDict<TexFont> fonts;
  QMap<QString, bool> fontNotFound;

  TexFont *currentFont;

  QRect bound;
};

#endif
