/*
 * This file is part of source for dviviewer, 
 * a DVI file viewer for Qtopia environment. 
 * 
 * Copyright (C) 2003  hn43o.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "filedialog.h"

#include <qlabel.h>
#include <qlayout.h>
#include <qlistbox.h>

DviFileDialog::DviFileDialog( QWidget *parent, const char *name ) : QDialog( parent, name, TRUE ) {
  labelDir = new QLabel( this );
  labelDir->setBackgroundColor( QColor(255, 255, 160 ) );
  listBoxFile = new QListBox( this );

  QVBoxLayout *l = new QVBoxLayout( this );
  l->addWidget( labelDir );
  l->addWidget( listBoxFile );

  connect( listBoxFile, SIGNAL( selected( int ) ), 
	   this, SLOT( fileSelected( int ) ) );
}

void DviFileDialog::setFile( QString d, QString f ) {
  dir.cd( d ); // This might fail.
  dirName = dir.absPath();
  labelDir->setText( dirName );

  listBoxFile->clear();
  dirList.clear();
  fileList.clear();

  dirList = dir.entryList( "*", QDir::Dirs );
  dirList.remove( "." );

  for( QStringList::Iterator it = dirList.begin() ; it != dirList.end(); ++it ) {
    listBoxFile->insertItem( *it + "/" );
  }

  fileList = dir.entryList( "*.dvi;*.DVI", QDir::Files );

  for( QStringList::Iterator it = fileList.begin() ; it != fileList.end(); ++it ) {
    QString s = *it;
    listBoxFile->insertItem( s );
    if ( s == f )
      listBoxFile->setCurrentItem( listBoxFile->count() - 1 );
  }
}

QString DviFileDialog::directory() {
  return dirName;
}

QString DviFileDialog::file() {
  int i = listBoxFile->currentItem();
  if ( i < ( int ) dirList.count() )
    return QString( "" );
  return ( fileList[i - dirList.count()] );
}

void DviFileDialog::fileSelected( int i ) {
  if ( i < ( int ) dirList.count() ) 
    setFile( dirName + "/" + dirList[i], "" );
  else
    accept();
}
