/*
 * This file is part of source for dviviewer, 
 * a DVI file viewer for Qtopia environment. 
 * 
 * Copyright (C) 2003  hn43o.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "jpfont.h"

#include <qimage.h>
#include <qpainter.h>
#include <qtextcodec.h>

#define FONTDIR ( "/home/QtPalmtop/share/jfm/" )

JpFont::JpFont() : TexFont() {
  codec = QTextCodec::codecForName( "EUCJP" );
}

bool JpFont::open( QString fileName ) {
  BinaryFile::open( fileName );
  readInfo();
  qDebug( "JpFont: %s %d", fileName.latin1(), file.isOpen() );
  return file.isOpen();
}

bool JpFont::openByName( QString name ) {
  isBold =  ( name.contains( "goth" ) );
  return open( QString( FONTDIR ) + name + QString( ".tfm" ) );
}

#define FFF ( ( ( float )( 1<<24 )*72.27 )/ ( float )_designSize )
uint JpFont::dx( uint code ) { // in inch, multiplied by 65536
  return width( code );
}

uint JpFont::width( uint code ) { // in inch, multiplied by 65536
  uint t = 0;
  if ( charType.contains( code ) )
    t = charType[code];
  return ( uint ) ( _width[charInfo[t].widthIndex]/FFF );
}

uint JpFont::height( uint code ) { // in inch, multiplied by 65536
  uint t = 0;
  if ( charType.contains( code ) )
    t = charType[code];
  return ( uint ) ( _height[charInfo[t].heightIndex]/FFF );
}

int JpFont::hoff( uint ) { // in inch, multiplied by 65536
  return 0;
}
int JpFont::voff( uint code ) { // in inch, multiplied by 65536
  return height( code );
}
uint JpFont::tfm( uint ) { 
  return 0;
}
uint JpFont::designSize() {
  return _designSize;
}

void JpFont::setDpi( uint d ) {
  dpi = d;
}

void JpFont::draw( QPainter *painter, int x, int y, uint code ) {
  char tmp[3];
  tmp[0] = ( ( code >> 8 ) | 0x80 );
  tmp[1] = ( ( code &0xff ) | 0x80 );
  tmp[2] = 0;
  QString s = codec->toUnicode( tmp );
  uint w = 
    ( uint ) ( ( dpi * _width[charInfo[0].widthIndex] )/( FFF*65536 ) );
  painter->setFont( QFont( painter->fontInfo().family(), w ) );
  QFontMetrics fm = painter->fontMetrics();
  painter->drawText( x, y + fm.height() - 1, s );
  if ( isBold )
    painter->drawText( x+1, y + fm.height() - 1, s );
}

#define CODE_YOKO 11
#define CODE_TATE 9

void JpFont::readInfo() { // actually, read everything.
  if ( !file.isOpen() )
    return;
  file.at( 0 );
  if ( file.size() < 28 ) { 
    qDebug( "JpFont: file too short to contain 'id' to 'np'" );
    close();
    return;
  }
  ushort code = getUInt2();
  if ( code != CODE_YOKO && code != CODE_TATE ) {
    qDebug( "JpFont: invalid JFM_ID" );
    close();
    return;
  }

  ushort nt, lf, lh, bc, ec, nw, nh, nd, ni, nl, nk, ng, np;

  nt = getUInt2();
  lf = getUInt2();
  lh = getUInt2();
  bc = getUInt2();
  ec = getUInt2();
  nw = getUInt2();
  nh = getUInt2();
  nd = getUInt2();
  ni = getUInt2();
  nl = getUInt2();
  nk = getUInt2();
  ng = getUInt2();
  np = getUInt2();

  // error check omitted...

  uint headerStart = 28;
  uint charTypeStart = headerStart + lh * 4;
  uint charInfoStart = charTypeStart + nt * 4;
  uint widthStart = charInfoStart + ( ec - bc + 1 ) * 4;
  uint heightStart = widthStart + nw * 4;
  uint depthStart = widthStart + nh * 4;
  uint italicStart = depthStart + nd * 4;

  file.at( headerStart + 4 );
  _designSize = getUInt();
  
  file.at( charTypeStart );
  for( int i=0; i < nt; i++ ) {
    ushort a = getUInt2(), b = getUInt2();
    charType.insert( a, b );
  }

  file.at( charInfoStart );
  charInfo.resize( ec - bc + 1 );
  for( int i=0; i < ec - bc + 1; i++ ) {
    uint a = getUInt();
    charInfo[i].widthIndex = a >> 24;
    charInfo[i].heightIndex = ( a >> 20 ) & 0xf;
    charInfo[i].depthIndex = ( a >> 16 ) & 0xf;
    charInfo[i].depthIndex = ( a >> 10 ) & 0x3f;
  }

  file.at( widthStart );
  _width.resize( nw );
  for( int i=0; i < nw; i++ )
    _width[i] = getUInt();

  file.at( heightStart );
  _height.resize( nh );
  for( int i=0; i < nh; i++ )
    _height[i] = getUInt();

  file.at( depthStart );
  _depth.resize( nd );
  for( int i=0; i < nd; i++ )
    _depth[i] = getUInt();

  file.at( italicStart );
  _italic.resize( ni );
  for( int i=0; i < ni; i++ )
    _italic[i] = getUInt();
}
